#include <iostream>
#include "Mesh.h"
#include "HarrisDetector.h"
#include "mex.h"

using namespace std;

void mexFunction(int nlhs, mxArray* plhs[], int nrhs, const mxArray* prhs[]){
	if(nrhs != 2){
		mexErrMsgTxt("Two inputs are expected");
	}
	
	const mxArray* strFilename;
	int length;
	char* modelFilename;
	char* optionFilename;
	
	strFilename = prhs[0];
	length = mxGetN(strFilename) + 1;
	modelFilename = (char*)mxCalloc(length, sizeof(char));
	mxGetString(strFilename, modelFilename, length);
	
	strFilename = prhs[1];
	length = mxGetN(strFilename) + 1;
	optionFilename = (char*)mxCalloc(length, sizeof(char));
	mxGetString(strFilename, optionFilename, length);
	
	Mesh mesh(modelFilename);
	
	Properties prop;
	prop.load(optionFilename);
	
	HarrisDetector hd(&mesh, &prop);
		
	vector<Vertex>	interestPoints;
	interestPoints.clear();
	Vertex* vertices = mesh.getVertices();
	int numVertex = mesh.getNumberVertices();
	
	hd.detectInterestPoints(interestPoints);
	
	mwSize dims[2]={interestPoints.size(),1};
	
	plhs[0] = mxCreateNumericArray(2, dims, mxINT32_CLASS, mxREAL);
	int* ptr = (int *)mxGetPr(plhs[0]);
	
	for(int i = 0; i < interestPoints.size(); i++)
		ptr[i] = interestPoints[i].getIndex() + 1;
	
	plhs[1] = mxCreateDoubleMatrix(numVertex, 1, mxREAL);
	double* array = mxGetPr(plhs[1]);
	
	for(int i = 0; i < numVertex; i++)
		array[i] = vertices[i].getResponse();
	
	mxFree(modelFilename);
	mxFree(optionFilename);
}
